"""
@author: Maxim Vavilin maxim.vavilin@kit.edu
"""

import numpy as np
import matplotlib.pyplot as plt

import repscat as rs




def get_identity(shape):

    identity = np.zeros(shape, dtype=complex)
    
    for idx0 in range(shape[0]):
        for idx1 in range(shape[2]):
            for idx2 in range(shape[4]):
                for idx3 in range(shape[6]):
                    identity[idx0, idx1, idx1, idx2, idx2, idx3, idx3] = 1
    
    return identity

def get_absorption(tmat):
    identity = get_identity(tmat.vals.shape)
    
    smat_vals = identity + tmat.vals
    
    s_dagger_s = np.einsum(
        'kxaycze,kxbydzf->kabcdef',
        np.conj(smat_vals), # transposed
        smat_vals, # (k,a,b,i,j,m,n)
        optimize = True
    )
    
    absorption_mat = identity - s_dagger_s
    

    trace = np.einsum(
            "kaaiimm,k->k",
            absorption_mat,  # (k,a,b,i,j,m,n)
            (4*np.pi)/tmat.k_list**2,
            optimize=True
        )
    
    # j_max = tmat_vals.shape[3]
    # normalization = (j_max+1)**2 - 1
    
    return trace


def plot(central_wavelength, xi_max, len_xi, radius, jay_max, ax=None):
    """Plot for article: Interaction cross-section as function of v/c"""

    if ax is None:
        fig, ax = plt.subplots()

    k0 = 2 * np.pi / central_wavelength
    rapidities = np.linspace(-xi_max, xi_max, len_xi)
    kays = k0 * np.exp(-rapidities)  # pulse seen inversely boosted
    
    tmat = rs.get_silicon_tmat(radius, jay_max, kays)

    absorption = get_absorption(tmat)

    linecolor = 'grey'

    velocities = np.tanh(rapidities)
    ax.plot(velocities, np.real(absorption)*1e-18, linestyle='-', color=linecolor, zorder=2.5)
    
    ax.set_ylabel(r"$\sigma^\text{aver}_\text{abs}$, $\text{m}^2$", color=linecolor)
    ax.tick_params(axis='y', labelcolor=linecolor)


    return ax
