"""
@author: Maxim Vavilin maxim.vavilin@kit.edu
"""

import numpy as np
import matplotlib.pyplot as plt

import repscat as rs


def plot(central_wavelength, xi_max, len_xi, radius, jay_max, ax=None):
    '''Plot for article: Interaction cross-section as function of v/c'''
    
    if ax is None:
        fig, ax = plt.subplots()
        
    k0 = 2*np.pi/central_wavelength
    rapidities = np.linspace(-xi_max, xi_max, len_xi)
    kays = k0*np.exp(-rapidities) # pulse seen inversely boosted
    tmat = rs.get_silicon_tmat(radius, jay_max, kays)
    
    norms =  np.einsum(
            "kabijmn,kabijmn,k->k",
            np.conj(tmat.vals),  # (k,a,b,i,j,m,n)
            tmat.vals,  # (k,a,b,i,j,m,n)
            (np.pi)/tmat.k_list**2, # using Karim and my convention
            optimize=True
        )
    
    linecolor = 'grey'
    
    velocities = np.tanh(rapidities)
    ax.plot(velocities, np.real(norms)*1e-18, linestyle='-', color=linecolor, zorder=2.5)


    ax.set_ylabel(r"$\sigma^\text{aver}_\text{sca}$, $\text{m}^2$", color=linecolor)
    ax.tick_params(axis='y', labelcolor=linecolor)


    
    return ax

